@@ -9,8 +9,8 @@ import imp
 import logging
 import os
 import re
-import sys
 import shutil
+import sys
 
 import py
 
@@ -153,7 +153,7 @@ def find_tags(directory):
             if url.startswith(BASE_ON_SERVER):
                 url = url.replace(BASE_ON_SERVER, BASE)
         yield (tag, name, url)
-        
+
 
 def override_global_constants(defaults_file):
     """Import the constants from the passed-in defaults file"""
@@ -167,6 +167,17 @@ def override_global_constants(defaults_file):
             logger.debug("Default for %s not overridden.", constant)
 
 
+def update_latest(latest, tag, name, url):
+    """Latest is a dict {package: version}"""
+    if name not in latest:
+        latest[name] = tag
+        return
+    current_version = LooseVersion(latest[name]).version
+    new_version = LooseVersion(tag).version
+    if new_version > current_version:
+        latest[name] = tag
+
+
 def main(defaults_file=None, specific_outfile=None):
     """Main method, called by bin/taglist
 
@@ -195,12 +206,24 @@ def main(defaults_file=None, specific_outfile=None):
             start = py.path.svnurl(BASE)
     else:
         start = py.path.svnurl(BASE)
+    latest = {}
     for tagsdir in find_tag_directories(start):
         for tag, name, url in find_tags(tagsdir):
             logger.info("Found tag %s for %s: %s",
                         tag, name, url)
+            update_latest(latest, tag, name, url)
             outfile.write(ENTRY % dict(tag=tag, name=name, url=url))
-    outfile.write(HTMLEND)
+    # New: prepare a section with latest versions.  Ideally this needs some
+    # refactoring to use a proper Jinja template, but I've got another thingy
+    # that needs the current behaviour...
+    latest_part = ['<h2><a name="latest">Latest versions</a></h2>', '<dl>']
+    packages = latest.keys()
+    packages.sort()
+    for package in packages:
+        version = latest[package]
+        latest_part.append('<dt>%s</dt><dd>%s</dd>' % (package, version))
+    latest_part.append('</dl>')
+    outfile.write(HTMLEND % {'latest_versions': '\n'.join(latest_part)})
     outfile.close()
     target = os.path.join(os.getcwd(), OUTFILE)
     if specific_outfile:
@@ -213,4 +236,3 @@ def main(defaults_file=None, specific_outfile=None):
     # make it world readable.
     os.chmod(target, 0644)
     logger.info("Generated list written to %s", target)
-
